package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageArrays.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author              Date
//  ------------  ------------------------     ---------------    ----------
//  0.0.0         Creation                      Dean Kalman       02/11/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** This class is the container object at the StorageSubsystem level for Array objects.  The only allowable 
  * children of Arrays are Array objects, as enforced by the add() method in this class.  
  */
public class StorageArrays extends RaidObject 
{
    static final long serialVersionUID = 7446769721470714216L;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoArrays"), 
        JCRMUtil.getNLSString("infoArraysSize"),
        JCRMUtil.getNLSString("infoArraysFreeSpace"),
        JCRMUtil.getNLSString("infoArraysLogDrives")
    };

    /** The StorageArrays constructor.
      */
    public StorageArrays()
    {
    }

    /** Removes a child Array from this RaidObject.
      * @param object  the Array to remove
      * @return  true if the specified Array was a child of this RaidObject, false otherwise.
      */
    public boolean remove(Array object)
    {
        return super.remove(object);
    }

    /** Returns an enumeration of the arrays in this container.
      * @return an enumeration of the arrays in this container
      */
    public Enumeration enumerateArrays()
    {
        Vector arrays = new Vector();

        for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
        {
            Object object = e.nextElement();
            if (object instanceof StorageGroup)
                arrays.addElement(object);
        }
        return arrays.elements();
    }

    /** This method satisfies ArrayCollectionIntf.  It returns a Vector of the
      * array objects managed by this adapter.
      * @param filter  If null, all arrays are returned.  If non-null, return arrays passed by the filter.
      * @return the Vector of array objects
      */
    public Vector getArrayCollection(StorageGroupFilterIntf filter)
    {
        Vector arrays = new Vector();
        for (Enumeration e = enumerateArrays(); e.hasMoreElements();)
        {
            StorageGroup array = (StorageGroup) e.nextElement();
            if ((filter == null) || (filter.accept(array)))
                arrays.addElement(array);
        }
        return arrays;
    }

    /** Updates the overall status for the arrays. 
      * @param recurse  true if this call should propagate down the config tree
      */
    public void updateOverallStatus(boolean recurse)
    {
        // Recursively update the status of all children
        if (recurse)
        {
           for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
              ((RaidObject) e.nextElement()).updateOverallStatus(recurse);
        }

        // No children
        if (getChildCount() == 0)
           setOverallStatus(NORMAL);

        // Check for all FATAL children condition
        else if (getChildrenByStatus(FATAL).size() == getChildCount())
           setOverallStatus(FATAL);

        // Check for all NORMAL children condition
        else if (getChildrenByStatus(NORMAL).size() == getChildCount())
           setOverallStatus(NORMAL);

        // In-between is FATAL
        else
           setOverallStatus(FATAL);
    }


    /** Changes the parent for this object.
      * @param object  The new parent StorageSubsystem
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageSubsystem parent, boolean addToParent)
    {
        setParent(parent);

        //Add ourselves to our new parent.
        if (addToParent)
            parent.add(this);
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();

        for (Enumeration children = enumerateArrays(); children.hasMoreElements();)
        {
            StorageGroup array = (StorageGroup) children.nextElement();
            v.addElement(new Object[]
                         { 
                             new StatusString(array.getDisplayID(), array.getOverallStatus()), 
                             String.valueOf(array.getSize()),
                             String.valueOf(array.getFreeSpace()), 
                             String.valueOf(array.getStorageLogicalDrivesContainer().getLogicalDriveCollection(null).size())
                         } );
        }

        return v;
    }           

    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the tree.
      * @return  the string used to identify this object in the tree
      */
    public String getDisplayName()
    {
        return JCRMUtil.getNLSString("arrays");
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        return(JCRMUtil.getNLSString("titleGUI"));
    }

    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @returns the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
        switch (getOverallStatus())
        {
            case NORMAL:    return "s_arrys1.gif";
            case CRITICAL:  return "s_arrys2.gif";
            case FATAL:     return "s_arrys3.gif";
            default:        return "s_arrys1.gif";
        }
    }

    /** Returns the XML representation of this object.  Can optionally include the standard
      * XML header string.  All children of this object are embedded as XML elements in the output.
      * @param includeHeader  whether to include the XML header
      * @param recurse  whether to embed this object's children
      * @return  the XML representation of this object
      */
    public String toXML(boolean includeHeader, boolean recurse)
    {
        StringBuffer xml = new StringBuffer();

        if (includeHeader)
            xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

        xml.append("<STORAGEARRAYS>").append(SEP);

        if (recurse)
            for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
                xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

        xml.append("</STORAGEARRAYS>").append(SEP);

        return xml.toString();
    }

    /** An Arrays can contain StorageGroup objects.
      * @param object  a StorageGroup
      */
    public void add(StorageGroup object)
    {
        super.add(object);
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * @param object  The object to compare to this one
      * @return  0 for equality, -1 if this object comes first, 1 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (object instanceof StorageEnclosure)
            return 1;
        else if (object instanceof StorageArrays)
            return 0;
        else if (object instanceof StorageLogicalDrives)
            return -1;
        else if (object instanceof StorageHotSpareDrives)
            return -1;
        else
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageArrays storageArrays = (StorageArrays) object;

        // not equal if either parent reference is null, but not both
        if ((getParent() == null) != (storageArrays.getParent() == null))
            return false;

        // not equal if parent is not null but equals fails    
        if ((getParent() != null) && !getParent().equals(storageArrays.getParent()))
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + ((getParent() == null) ? 0 : getParent().hashCode());
        return result;
    }
}
