package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageDevice.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK           02/07/02
//

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

public abstract class StorageDevice extends RaidObject 
{
    static final long serialVersionUID = 5412898114009887308L;

    private int     slot;
    private String  manufacturer;
    private String  productId;
    private String  serialNumber;
    private String  firmwareLevel;
    private String  partNumber;
    private boolean pfa;
    private long    capacity;
    private StorageEnclosure enclosure;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoPhysDriveName"), 
        JCRMUtil.getNLSString("infoPhysDriveValue"),
    };

    /** The StorageDevice constructor.
      * @param slot - the slot this drive is in
      * @param productInfo
      * @param firmwareLevel 
      * @param pfa
      */
    public StorageDevice (int slot,
                          ProductInfo productInfo,
                          String firmwareLevel,
                          boolean pfa,
                          long capacity)
    {
        this.slot = slot;
        this.manufacturer = productInfo.getManufacturer();
        this.productId = productInfo.getProductId();
        this.serialNumber = productInfo.getSerialNumber();
        this.firmwareLevel = firmwareLevel;
        this.partNumber = productInfo.getPartNumber();
        this.pfa = pfa;
        this.capacity = capacity;
    }

    /** Get the slot this device is installed in
      * @return the slot this device is installed in
      */
    public int getSlot() 
    {
        return slot;
    }

    /** Get the manufacturer of this device
      * @return the manufacturer of this device
      */
    public String getManufacturer()
    {
        return manufacturer;
    }

    /** Get the product id of this device
      * @return the product id of this device
      */
    public String getProductId()
    {
        return productId;
    }

    /** Get the serial number of this device
      * @return the serial number of this device
      */
    public String getSerialNumber()
    {
        return serialNumber;
    }

    /** Get the firmware level of this device
      * @return the firmware level of this device
      */
    public String getFirmwareLevel()
    {
        return firmwareLevel;
    }

    /** Get the part number of this device
      * @return the part number of this device
      */
    public String getPartNumber()
    {
        return partNumber;
    }

    /** Get the pfa of this device
      * @return the pfa of this device
      */
    public boolean getPfa()
    {
        return pfa;
    }

    /** Returns this drive's capacity.
      * @return this drive's capacity
      */
    public final long getCapacity()
    {
        return capacity;
    }
    
    /** Get the enclosure this device is installed in
      * @return the enclosure this device is installed in, or null if not installed in an enclosure
      */
    public StorageEnclosure getEnclosure()
    {
        return enclosure;
    }

    /** Changes the parent for this object.
      * @param object  The new parent Enclosure
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageEnclosure parent, boolean addToParent)
    {
       setParent(parent);

       //Add ourselves to our new parent.
       if (addToParent)
          parent.add(this);

       enclosure = parent;
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();

        v.addElement(new Object[] { new ToolTipString("infoStorageDeviceSlot"), String.valueOf(getSlot())});
        v.addElement(new Object[] { new ToolTipString("infoStorageDevicePFA"), String.valueOf(getDisplayPfa())});
        v.addElement(new Object[] { new ToolTipString("infoStorageDeviceModel"), String.valueOf(getProductId())});
        v.addElement(new Object[] { new ToolTipString("infoStorageDeviceVendor"), String.valueOf(getManufacturer())});
        v.addElement(new Object[] { new ToolTipString("infoStorageDeviceFirmware"), String.valueOf(getFirmwareLevel())});
        v.addElement(new Object[] { new ToolTipString("infoStorageDeviceSerial"), String.valueOf(getSerialNumber())});
        return v;
    } 

    /** Returns the displayable pfa information.
      * @return the displayable pfa information      
      */
    public String getDisplayPfa()
    {
        if (getPfa() == true)
            return JCRMUtil.getNLSString("yes");
        else
            return JCRMUtil.getNLSString("no");
    }
    
    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = {String.valueOf(getSlot())};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleHardDriveInfo", args));
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageDevice device = (StorageDevice) object;

        // Start with lowest level to make things fast
        if (getSlot() != device.getSlot())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * Ordering of StorageDevice object is based on deviceID.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (object instanceof StorageController)
            return 1;

        if (!(object instanceof StorageDevice))
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
        else
        {
            StorageDevice device = (StorageDevice) object;

            if (getSlot() < device.getSlot())
                return -1;
            else if (getSlot() > device.getSlot())
                return 1;
            else return(getSlot() - device.getSlot());
        }
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getSlot();
        return result;
    }

}
