package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageSubsystem.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK            02/04/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** The storage subsystem. A storage subsystem contains a storage enclosure and its associated physical and
  * logical devices 
  */
public abstract class StorageSubsystem extends RaidObject 
{
    static final long serialVersionUID = -4857015652842261552L;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoSubsystemName"), 
        JCRMUtil.getNLSString("infoSubsystemValue"),
    };                                                                                      

    private int         id;
    private boolean     inBand;
    private String      hostName;
    private int         status;
    private int         controllerCount;
    private int         arrayCount;
    private int         logicalDriveCount;
    private int         physicalDriveCount;
    private int         displayId = NOT_DEFINED;
    private boolean     supported = true;
    private RaidSystem  system;
    private StorageArrays         storageArrays;
    private StorageLogicalDrives  storageLogicalDrives;
    private StorageHotSpareDrives storageHotSpareDrives;

    /** The StorageSubsystem constructor.
      * @param id - the id of this subsystem, based on the subsystem discovery order
      * @param inBand - set true if in band connection to subsystem
      * @param hostName - the hostname of the controller or agent that can access this subsystem
      * @param status - the status of the storage subsystem. Status values are defined in this class. 
      * @param controllerCount - the number of storage controllers in this subsystem
      * @param arrayCount - the number of arrays in this subsystem
      * @param logicalDriveCount - the number of logical drives in this subsystem
      * @param physicalDriveCount - the number of physical drives in this subsystem
      */
    public StorageSubsystem(int id, 
                            boolean inBand,
                            String hostName,
                            int status, 
                            int controllerCount,
                            int arrayCount,
                            int logicalDriveCount,
                            int physicalDriveCount)
    {
        this.id = id;
        this.inBand = inBand;
        this.hostName = hostName;
        this.status = status;
        this.controllerCount = controllerCount;
        this.arrayCount = arrayCount;
        this.logicalDriveCount = logicalDriveCount;
        this.physicalDriveCount = physicalDriveCount;

        // Add the arrays container 
        storageArrays = new StorageArrays();
        add(storageArrays);

        // Add the logical drives container 
        storageLogicalDrives = new StorageLogicalDrives();
        add(storageLogicalDrives);

        // Add the hot spare drives container
        storageHotSpareDrives = new StorageHotSpareDrives();
        add(storageHotSpareDrives);
    }

    /** Returns this subsystem's id.
      * @return this subsystem's id
      */
    public int getId()
    {
        return id;
    }

    /** Returns this subsystem's inBand status.
      * @return this subsystem's inBand status
      */
    public boolean getInBand()
    {
        return inBand;
    }

    /** Returns the hostname that can see this subsystem.
      * @return the hostname that can see this subsystem
      */
    public String getHostName()
    {
        return hostName;
    }

    /** Returns this subsystem's display id.
      * @return this subsystem's display id
      */
    public String getDisplayID()
    {
        if (displayId != NOT_DEFINED)
            return String.valueOf(displayId);
        else
            return String.valueOf(id+1);
    }

    /** Sets this subsystem's display id.
      * @param set this subsystem's display id
      */
    public void setDisplayId(String displayId)
    {
        this.displayId = new Integer(displayId).intValue();
    }
    
    /** Sets this subsystem's id.
      * @param set this subsystem's id
      */
    public void setId(int id)
    {
        this.id = id;
    }


    /** Returns this subsystem's status.
      * @return this subsystem's status
      */
    public int getStatus()
    {
        return status;
    }

    /** Returns the controller count.
      * @return the controller count
      */
    public int getControllerCount()
    {
        return controllerCount;
    }

    /** Returns the array count.
      * @return the array count
      */
    public int getArrayCount()
    {
        return arrayCount;
    }

    /** Returns the logical drive count.
      * @return the logical drive count
      */
    public int getLogicalDriveCount()
    {
        return logicalDriveCount;
    }

    /** Returns the physical drive count.
      * @return the physical drive count
      */
    public int getPhysicalDriveCount()
    {
        return physicalDriveCount;
    }

    /** Returns this subsystem's raid system.
      * @return this subsystem's raid system
      */
    public RaidSystem getRaidSystem()
    {
        return system;
    }

    /** Returns the storage arrays container
      * @return the storage arrays container
      */
    public StorageArrays getStorageArraysContainer() 
    {
        return storageArrays;
    }

    /** Returns the logical drives container
      * @return the logical drives arrays container
      */
    public StorageLogicalDrives getStorageLogicalDrivesContainer() 
    {
        return storageLogicalDrives;
    }

    /** Returns the hot spare drives container
      * @return the hot spare drives container
      */
    public StorageHotSpareDrives getStorageHotSpareDrivesContainer() 
    {
        return storageHotSpareDrives;
    }

    /** Set true if this subsystem supported.
      * @param value set true if this subsystem supported, false otherwise
      */
    protected void setSupported(boolean value)
    {
        supported = value;
    }
    
    /** Returns true if this subsystem is supported, false otherwise.
      * @return true if this subsystem is supported, false otherwise
      */
    public boolean getSupported()
    {
        return supported;
    }


    /** This method returns a Vector of the array objects contained in this object.
      * @param filter  If null, all arrays are returned.  If non-null, return arrays passed by the filter.
      * @return the Vector of array objects
      */
    public Vector getArrayCollection(StorageGroupFilterIntf filter)
    {
        return getStorageArraysContainer().getArrayCollection(filter); 
    }

    /** This method returns a Vector of the enclosure objects contained in this object.
      * @param filter  If null, all enclosures are returned.  If non-null, return enclosures passed by the filter.
      * @return the Vector of enclosure objects
      */
    public Vector getEnclosureCollection(StorageEnclosureFilterIntf filter)
    {
        Vector enclosures = new Vector();
        for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
        {
            Object object = e.nextElement();
            if (object instanceof StorageEnclosure)
            {
                StorageEnclosure enclosure = (StorageEnclosure) object;
                if ((filter == null) || (filter.accept(enclosure)))
                    enclosures.addElement(enclosure);
            }
        }
        return enclosures;
    }

    /** This method returns a Vector of the controller objects contained in this object.
      * @param filter  If null, all controllers are returned.  If non-null, return controllers passed by the filter.
      * @return the Vector of controller objects
      */
    public Vector getContollerCollection(StorageControllerFilterIntf filter)
    {
        Vector controllers = new Vector();
        for (Enumeration e = getEnclosureCollection(null).elements(); e.hasMoreElements();)
        {
            StorageEnclosure enclosure = (StorageEnclosure)e.nextElement();
            for (Enumeration f = enclosure.enumerateChildren(); f.hasMoreElements();)
            {
                Object object = f.nextElement();
                if (object instanceof StorageController)
                {
                    StorageController controller = (StorageController) object;
                    if ((filter == null) || (filter.accept(controller)))
                        controllers.addElement(controller);
                }
            }
        }
        return controllers;
    }

    /** This method returns a Vector of the device objects contained in this object.
      * @param filter  If null, all device are returned.  If non-null, return devices passed by the filter.
      * @return the Vector of device objects
      */
    public Vector getDeviceCollection(StorageDeviceFilterIntf filter)
    {
        Vector devices = new Vector();
        for (Enumeration e = getEnclosureCollection(null).elements(); e.hasMoreElements();)
        {
            StorageEnclosure enclosure = (StorageEnclosure)e.nextElement();
            for (Enumeration f = enclosure.enumerateChildren(); f.hasMoreElements();)
            {
                Object object = f.nextElement();
                if (object instanceof StorageDevice)
                {
                    StorageDevice device = (StorageDevice) object;
                    if ((filter == null) || (filter.accept(device)))
                        devices.addElement(device);
                }
            }
        }
        return devices;
    }

    /** Updates the overall status of this object based on its fields. 
      * @param recurse  true if this call should propagate down the config tree
      */
    public void updateOverallStatus(boolean recurse)
    {
        // Recursively update the status of all children
        if (recurse)
        {
            for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
                ((RaidObject) e.nextElement()).updateOverallStatus(recurse);
        }
        setOverallStatus(getStatus());
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();

        if (getInBand() == false)
            v.addElement(new Object[] { new ToolTipString("infoSubsystemManagementType"), JCRMUtil.getNLSString("valueSubsystemManagementTypeDirect")});
        else 
        {
            if (getHostName() != null)
            {
                Object[] args = { getHostName() };
                v.addElement(new Object[] { new ToolTipString("infoSubsystemManagementType"), JCRMUtil.makeNLSString("valueSubsystemManagementTypeHosts", args)});
            }
        }
        if (getSupported())
        {
            v.addElement(new Object[] { new ToolTipString("infoSubsystemControllerCount"), String.valueOf(getControllerCount())});
            v.addElement(new Object[] { new ToolTipString("infoSubsystemArrayCount"), String.valueOf(getArrayCount())});
            v.addElement(new Object[] { new ToolTipString("infoSubsystemLogicalDriveCount"), String.valueOf(getLogicalDriveCount())});
            v.addElement(new Object[] { new ToolTipString("infoSubsystemPhysicalDriveCount"), String.valueOf(getPhysicalDriveCount())});
        }
        return v;
    }

    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = { getDisplayID()};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleSubsystemInfo", args));
    }

    /** Changes the parent for this object.
      * @param object  The new parent RaidSystem
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(RaidSystem parent, boolean addToParent)
    {
        setParent(parent);
        system = parent;

        if (addToParent)
            parent.add(this);
    }

    /** A StorageSubsystem can contain StorageEnclosure objects.
      * @param object a StorageEnclosure
      */
    public void add(StorageEnclosure object)
    {
        super.add(object);
    }

    /** A StorageSubsystem contains a StorageArrays object.
      * @param object a StorageArrays object
      */
    public void add(StorageArrays object)
    {
        super.add(object);
    }

    /** A StorageSubsystem contains a StorageLogicalDrives object.
      * @param object a StorageLogicalDrives object
      */
    public void add(StorageLogicalDrives object)
    {
        super.add(object);
    }

    /** A StorageSubsystem contains a StorageHotSpareDrives object.
      * @param object a StorageHotSpareDrives object
      */
    public void add(StorageHotSpareDrives object)
    {
        super.add(object);
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageSubsystem storageSubsystem = (StorageSubsystem) object;

        // Compare all relevant fields
        if (getId() != storageSubsystem.getId())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (!(object instanceof StorageSubsystem))
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
        else
        {
            StorageSubsystem storageSubsystem = (StorageSubsystem) object;
            int displayId = new Integer(getDisplayID()).intValue();
            int compareDisplayId = new Integer(storageSubsystem.getDisplayID()).intValue();
            return(displayId - compareDisplayId);
        }
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getId();
        return result;
    }
}

