package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocHLogicalDrive.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/16/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/**
 * Represents a Hierarchical Logical Drive.
 * Created using HLogDrv.java as the base design
 */

public class IrocHLogicalDrive extends HLogicalDrive implements Constants, Cloneable
{
   static final long serialVersionUID = -4971733146685208622L;

   /** Creates an IrocHLogicalDrive object with the given parameters.
     * @param  adapterIN                The adapter who owns this HLD
     * @param  arrayIN                  The array to which this HLD belongs
     * @param  logicalDriveIDIN         The logical drive ID of this HLD
     * @param  stateIN                  State of the logical drive
     * @param  raidLevelIN              Raid Level of this HLD
     * @param  dataSpaceIN              Amount of datay space in MB
     * @param  paritySpaceIN            Amount of parity space in MB
     * @param  blockedAccessIN          True if this logical drive is blocked
     * @param  writeCacheModeIN         Write cache mode
     * @param  lastDriveInArray         True if this drive is the last in its array
     * @param  stateFlags               Additonal state bits, such as expanding, compacting, etc.
     * @see    com.ibm.sysmgt.raidmgr.dataproc.BasicLogicalDrive
     * @see    com.ibm.sysmgt.raidmgr.dataproc.LogicalDrive
     */
   public IrocHLogicalDrive(Adapter adapterIN, 
                            Array arrayIN, 
                            int logicalDriveIDIN, 
                            int stateIN, 
                            int raidLevelIN,
                            int dataSpaceIN, 
                            int paritySpaceIN, 
                            boolean blockedAccessIN,
                            int writeCacheModeIN,
                            boolean lastDriveInArrayIN)
   {
      super(adapterIN, 
            arrayIN, 
            logicalDriveIDIN, 
            stateIN, 
            raidLevelIN, 
            dataSpaceIN, 
            paritySpaceIN, 
            "",                  // date created not available
            blockedAccessIN, 
            writeCacheModeIN, 
            -1,                  // merge group not supported
            false,               // shared drives not supported
            true,                // drive exists
            false,               // temp write through not supported
            lastDriveInArrayIN,
            null);
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      Vector actions = new Vector();
      actions.addElement(new HelpNowAction(this));
      actions.addElement(new NullAction());
      //actions.addElement(new UnblockLogDriveAction(this));
      actions.addElement(new InitLogDriveAction(this));
      actions.addElement(new SyncLogDriveAction(this));
      actions.addElement(new DeleteLogDriveAction(this, false /*compositeAction*/));
      actions.addElement(new NullAction());
      actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_HLOG_DRV, getArrayID(), getID()));
      setActions(actions);

      return actions;
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();

      String stateString;
      switch (getState())
      {
         case OFL:  stateString = JCRMUtil.getNLSString("infoLogDriveOffline");     break;
         case OKY:  stateString = JCRMUtil.getNLSString("infoLogDriveOkay");        break;
         case CRT:  stateString = JCRMUtil.getNLSString("infoLogDriveCritical");    break;
         default:   stateString = JCRMUtil.getNLSString("unknown"); 
      }

      if (isBlocked())
         stateString = JCRMUtil.getNLSString("infoLogDriveBlocked");

      v.addElement(new Object[] { new ToolTipString("infoLogDriveNum"), getDisplayID()});
      v.addElement(new Object[] { new ToolTipString("infoArrayLetter"), String.valueOf(Array.IDToLetter(getArrayID()))});
      v.addElement(new Object[] { new ToolTipString("infoLogDriveState"), new StatusString(stateString, getOverallStatus())});
      v.addElement(new Object[] { new ToolTipString("infoLogDriveRaid"), getDisplayRaidLevel()});
      v.addElement(new Object[] { new ToolTipString("infoLogDriveData"), String.valueOf(getDataSpace())});
      v.addElement(new Object[] { new ToolTipString("infoLogDriveParity"), String.valueOf(getParitySpace())});

      return v;
   }

   /** Returns a clone of this object.
     * @return the clone
     */
   public Object clone()
   {
      return super.clone();
   }
}
